//-------------------------------------------//
//  LSPTweak2k by MrLoathsome.    8/11/2018  //
//-------------------------------------------//

class LMT extends Mutator config(LSPTweak2k);

var() config bool bDebug;
var() config float LMTRate;

struct DefSPType
{
	var string SPClass;
	var bool SPAdjust;
	var int SPHealth;
	var float SPDamageScaling;
	var string SPNameArticle;
	var string SPMenuName;
	var string SPRangedProj;
	var int SPSpawnCnt;
};

var() config DefSPType SPCList[2048];   // Raise this number and recompile if your ini file fills up...

function PreBeginPlay()
{
	local int i;
	local ScriptedPawn TP;

	foreach allactors(class'ScriptedPawn', TP)  // check all scriptedpawns that may be on the map by default
	{
		CheckMonster(TP);
	}
	for (i=0; SPCList[i].SPClass != ""; i++)
	{
		// count array...
	}
	log("LSPTweak -- Sorting SPCList array.  Count = "@i);
	if (i > 1) SortArray(0, i-1);
	SaveConfig();
	Spawn(class'MyWatcher',Self);  // spawn our watcher class that checks any other scriptedpawns that spawn during the game
}

function bool HandleEndGame()
{
	SaveConfig();
	if (bDebug) log("LMT - HandleEndGame() & SaveConfig() called."@Level.TimeSeconds);
	if ( NextMutator != None )
		return NextMutator.HandleEndGame();
	return false;
}

function Timer()
{
	SaveConfig();
	if (bDebug) log("LMT - Timer().   SaveConfig() called.  ->"@Level.TimeSeconds);
}

function bool CheckMonster(ScriptedPawn SP)  //  Called by watcher class.  Ajusts SP and returns false if SP exists in ini.   Else adds SP to ini file with class default values
{
	local int i;

	for (i=0; SPCList[i].SPClass != ""; i++)
	{
		if (class<ScriptedPawn>(DynamicLoadObject(SPCList[i].SPClass, class'Class')) == SP.Class)  // SP exists in ini file.   Adjust and/or count and return
		{
			SPCList[i].SPSpawnCnt++;
			if (LMTRate != 0.0) SetTimer(LMTRate, False);
			if (!SPCList[i].SPAdjust) return false;
			SP.Health = SPCList[i].SPHealth;
			SP.DamageScaling = SPCList[i].SPDamageScaling;
			if (SPCList[i].SPNameArticle != "") SP.NameArticle = SPCList[i].SPNameArticle;
			if (SPCList[i].SPMenuName != "") SP.MenuName = SPCList[i].SPMenuName;
			if (SPCList[i].SPRangedProj != "None") SP.RangedProjectile = class<Projectile>(DynamicLoadObject(SPCList[i].SPRangedProj, class'Class'));
			return false;
		}
	}
	SPCList[i].SPClass = String(SP.Class);                       // New ScriptedPawn detected.  Add to ini file with default values
	log("LMT - New ScriptedPawn Class Detected:"@SP.Class);
	SPCList[i].SPHealth = SP.default.Health;
	SPCList[i].SPDamageScaling = SP.default.DamageScaling;
	SPCList[i].SPNameArticle = SP.default.NameArticle;
	SPCList[i].SPMenuName = SP.default.MenuName;
	SPCList[i].SPRangedProj = String(SP.default.RangedProjectile);
	SPCList[i].SPSpawnCnt++;
	SaveConfig();
	return true;
}

function SortArray(Int Low, Int High) // Sortage - Adapted from the Wiki.beyondunreal.com example to sort an array of structures from High to Low based on a variable in the structure
{
	//  low is the lower index, high is the upper index
	//  of the region of array a that is to be sorted
 
	local int i,j;
	local int x;
	local DefSPType Temp;  //  DefSPType is the global structure type used by this class in this example.   Adjust as needed for your array/structure/class.

	i = Low;
	j = High;
	x = SPCList[(Low+High)/2].SPSpawnCnt;  // Sort by structure variable SPSpawnCnt for this example

	do
	{
		while (SPCList[i].SPSpawnCnt > x)  // Change to < X for Low to High sort
			i += 1;
		while (SPCList[j].SPSpawnCnt < x)  // Change to > X for Low to High sort
			j -= 1;
			if (i <= j)
			{
				Temp = SPCList[i];
				SPCList[i] = SPCList[j];
				SPCList[j] = Temp;
				i += 1;
				j -= 1;
			}
	} until (i > j);
 
	//  recursion
	if (low < j)
		SortArray(low, j);
	if (i < high)
		SortArray(i, high);
}

defaultproperties
{
      LMTRate=0.0
}
